#
# spec file for package libpulp
#
# Copyright (c) 2022 SUSE LLC
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via https://bugs.opensuse.org/
#


Name:           libpulp
Version:        0.3.1
Release:        1.1
Summary:        Userspace live patching library and tools
License:        LGPL-2.1-only
Group:          Productivity/Security
URL:            https://github.com/suse/libpulp
Source0:        %{name}-%{version}.tar.gz
Source1:        rpm-helper
Source2:        macros.userspace-livepatch
Source99:       libpulp.rpmlintrc
# Required to hardlink identical files.
BuildRequires:  fdupes
# Required to run the tests.
BuildRequires:  gcc-c++
# Required to build the tools, which are needed to run the tests.
BuildRequires:  libjson-c-devel
BuildRequires:  libelf-devel
BuildRequires:  python3-pexpect
BuildRequires:  python3-psutil
BuildRequires:  libseccomp-devel
# Only available for these architectures.
ExclusiveArch:  x86_64

%description
Library and tools for user space live patching.

%package -n libpulp0
Summary:        User space live patching library
Group:          System/Libraries

%description -n libpulp0
Libpulp is a library (and a framework) that enables live patching of
user space libraries.

This package contains the runtime files.

%package tools
Summary:        User space live patching tools
Group:          System/Management

%description tools
This package contains the tools to apply user-space live patches.

# Disable LTO for libpulp, as it is currently not supported.
%define _lto_cflags %{nil}

%prep
%autosetup -p1

%build
%configure
%make_build

%check
%make_build check

%install
%make_install
install -D -m0755 %{SOURCE1} %{buildroot}%{_prefix}/lib/userspace-livepatch/rpm-helper
install -D -m0644 %{SOURCE2} %{buildroot}%{_prefix}/lib/rpm/macros.d/macros.userspace-livepatch

# Convert identical files into hardlinks.
%fdupes %{buildroot}/%{_prefix}
# Remove .la and .so files.  libpulp.so is not supposed to be linked
# against any programs or libraries, but LD_PRELOAD'ed, so do not
# distribute it, not even in the devel package.
find %{buildroot}/%{_prefix} -name libpulp.la -delete
find %{buildroot}/%{_prefix} -name libpulp.so -delete

%post -n libpulp0 -p /sbin/ldconfig
%postun -n libpulp0 -p /sbin/ldconfig

%files -n libpulp0
%{_libdir}/lib*.so.*
%doc README.md
%license LICENSE

%files tools
%{_bindir}/*
%{_mandir}/*/*
%dir %{_prefix}/lib/userspace-livepatch
%{_prefix}/lib/userspace-livepatch/*
%{_prefix}/lib/rpm/*
%license LICENSE

%changelog
* Mon Oct 23 2023 Giuliano Belinassi <gbelinassi@suse.de>
- Update package with libpulp-0.3.1:
  * Add timestamp information on `ulp patches`.
* Tue Jun 27 2023 Giuliano Belinassi <giuliano.belinassi@suse.com>
- Update package with libpulp-0.3.0:
  * Add support for processes with blocked mprotect (process launched by
    systemd, for example) (bsc#1210224, jsc#PED-2877).
  * Add support for processes which chroots into /proc.
  * Supports livepathcing all processes in SLE.
* Fri Jun 16 2023 Giuliano Belinassi <giuliano.belinassi@suse.com>
- Update package with libpulp-0.2.11:
  * Avoid warning on symbol read of processes which user do not have access.
  * Fix a bug in livepatch installation counting.
  * Fix a warning message of library not loaded when reverting all patches when
    the library is loaded.
  * Fix a crash when `patches` is called with invalid PID.
  * Enable batch processing for patching a single process via PID.
* Thu Apr 13 2023 Giuliano Belinassi <giuliano.belinassi@suse.com>
- Update package with libpulp-0.2.10:
  * Fix typo which makes write_bytes fallback to ptrace mode when vm_writev is
    available.
  * Detect when mprotect is blocked by seccomp (process launched by systemd,
    for example) and disable livepatching in the process (bsc#1210224,
    jsc#PED-2877).
* Thu Mar  9 2023 Giuliano Belinassi <giuliano.belinassi@suse.com>
- Update package with libpulp-0.2.9:
  * Add mechanism to enable or disable livepatching based or environment variables
    and in the new command `ulp set_patchable` (jsc#PED-2877).
  * Change `patch already applied` message from error to skipped.
* Thu Feb 23 2023 Giuliano Belinassi <giuliano.belinassi@suse.com>
- Update package with libpulp-0.2.8:
  * Minor code refactoring.
  * Fixed a bug where libpulp rejected correct ELF files as library input.
  * Fixed a file descriptor leak when -check-stack is passed to ulp.
  * Fixed a bug where ulp did not shown libcrypto.so.1.1 as a livepatchable library (bsc#1208575)
* Mon Jan  2 2023 Giuliano Belinassi <giuliano.belinassi@suse.com>
- Update package with libpulp-0.2.7:
  * Add support to library to JSON library dumps, Removing any requirement of
    adding the original library .so file into the livepatch build tarball.
  * Update the ulp post hook script for transactional systems (jsc#PED-1078).
  * Add `setup_package.sh` as part of libpulp tools.
* Thu Nov 10 2022 Giuliano Belinassi <giuliano.belinassi@suse.com>
- Update package with libpulp-0.2.6
  * Add new `-R` option to specify a prefix root for livepatches
    (jsc#PED-1078).
* Thu Aug 18 2022 Giuliano Belinassi <giuliano.belinassi@suse.com>
- Update package with libpulp-0.2.5.
  * Fix ulp tool not patching on highly stressed environments. The reason behind
    it is that a 10s timeout was not enough depending of how stressed the
    machine is. Worse cases when libpulp is running in a VM (bsc#1200316).
  * Fix HANA testcase failures (bsc#1200129).
  * Add support for searching for patches recursively. Previous versions only
    searched on the path specified, ignoring subdirectories.
  * Improve patching performance. Previous version took up to 20s ~ 25s to
    patch 4000 processes. This version reduces this time to 6s. The way this
    is done is reducing ptrace calls and switching to process_vm_readv/writev
    when possible, and moving process discovery to a different thread.
* Fri Jun 24 2022 Giuliano Belinassi <giuliano.belinassi@suse.com>
- Fix ulp tool not patching on high process count (bsc#1200316).
- Implement a timeout feature in case of deadlocks.
* Thu Jun 23 2022 Giuliano Belinassi <giuliano.belinassi@suse.com>
- Fix ulp tool crashing on high process count (bsc#1200316).
- Avoid parsing /proc/<pid>/comm when not needed.
* Mon Jun 13 2022 Giuliano Belinassi <giuliano.belinassi@suse.com>
- Update package with libpulp-0.2.4.
- Fix dlsym interposition changing program behaviour (bsc#1200129)
- Fix free call of mmap'ed buffers (bsc#1200129)
- Fix error message when user has no permission to open livepatch.
* Thu May 12 2022 Giuliano Belinassi <giuliano.belinassi@suse.com>
- Update package with libpulp-0.2.3 (jsc#SLE-20049).
- Add support for endbr64 instructions on function beginning.
- Fix use-after-free bug.
- Fix compilation in Tumbleweed.
* Mon May  2 2022 Giuliano Belinassi <giuliano.belinassi@suse.com>
- Update package with libpulp-0.2.2 (jsc#SLE-20049).
- Use colored output by default (disable with --color=no)
- Packer now reports errors in .dsc in a GCC 5+ fashion.
- Trigger now has a summarized mode (disable with -v)
* Tue Apr 12 2022 Giuliano Belinassi <giuliano.belinassi@suse.com>
- Update package with libpulp-0.2.1 (jsc#SLE-20049).
- Fix base address load of non-library variables in target process.
- Dump references information on `ulp dump`.
* Wed Mar 30 2022 Giuliano Belinassi <giuliano.belinassi@suse.com>
- Update package with libpulp-0.2.0 (jsc#SLE-20049).
- Embed metadata (.ulp) into livepatch container (.so).
* Fri Mar 25 2022 Libor Pechacek <lpechacek@suse.com>
- Add patch build macros and deployment scripts. (jsc#SLE-20049)
* Tue Feb 22 2022 Giuliano Belinassi <giuliano.belinassi@suse.com>
- Update package with libpulp-0.1.1 (jsc#SLE-20049).
- Add new command `ulp livepatchable` to check if a library is livepatchable.
* Wed Feb 16 2022 Giuliano Belinassi <giuliano.belinassi@suse.com>
- Update package with newest libpulp features (jsc#SLE-20049).
- Fix a bug which causes the ulp tool to fail if itself was loaded with libpulp.
* Fri Jan 28 2022 Giuliano Belinassi <giuliano.belinassi@suse.com>
- Update package with newest libpulp features (jsc#SLE-20049).
- Fix --revert-all when no wildcards are provided.
- Fix batch processing of .rev files.
- Disable lto when building libpulp.
* Fri Nov 12 2021 Giuliano Belinassi <giuliano.belinassi@suse.com>
- Update libpulp with ulp_apply in trigger.
* Fri Oct 22 2021 Giuliano Belinassi <giuliano.belinassi@suse.com>
- Remove gcc9 as build requirement.
* Tue Oct 12 2021 Libor Pechacek <lpechacek@suse.com>
- Add libpulp.rpmlintrc to the sources. (jsc#SLE-20049)
- Refresh the .spec file with spec-cleaner.
* Tue Oct  5 2021 Giuliano Belinassi <giuliano.belinassi@suse.com>
- Update libpulp .tar.gz package.
* Mon Sep 27 2021 Giuliano Belinassi <gbelinassi@suse.de>
- Update libpulp .tar.gz package.
- Remove gcc9-PIE from libpulp.spec, as it is not provided anymore.
* Mon Feb  3 2020 Gabriel F. T. Gomes <gagomes@suse.de>
- Initial package.
